use std::sync::Arc;

use axum::Router;
use axum::routing::get;
use sqlx::mysql::MySqlPoolOptions;
use tokio::signal;
use tracing_appender::{rolling, non_blocking};
use tracing_subscriber::{EnvFilter, fmt, layer::SubscriberExt, util::SubscriberInitExt};

use axum_sqlx::{config::AppConfig, handler, model::state::AppState};


fn init_tracing_logger() {
    // 设置日志级别 根据RUST_LOG
    let env_filter = EnvFilter::try_from_default_env().unwrap_or_else(|_| EnvFilter::new("info"));
    // 输出到文件中（按天分割）
    let file_appender = rolling::daily("logs", "app.log");
    // 非阻塞
    let (non_blocking_appender, _guard) = non_blocking(file_appender);
    // 构建layer
    let file_layer = fmt::layer()
        .with_ansi(false)
        .with_writer(non_blocking_appender);

    tracing_subscriber::registry()
        .with(env_filter)
        .with(fmt::layer())
        .with(file_layer)
        .init();
}


#[tokio::main]
async fn main() {
    // 解析 .env 文件
    dotenvy::dotenv().expect("解析.env文件失败");

    init_tracing_logger();

    let cfg = AppConfig::new()
        .map_err(|e| tracing::error!("初始化配置失败：{}", e.to_string()))
        .unwrap();

    let pool = MySqlPoolOptions::new()
        .max_connections(cfg.mysql.max_cons)
        .connect(&cfg.mysql.dsn)
        .await
        .map_err(|e| tracing::error!("数据库连接失败：{}", e.to_string()))
        .unwrap();

    let app = Router::new()
        .route("/", get(handler::index))
        .route("/detail/:id", get(handler::detail))
        .route("/add", get(handler::add_ui).post(handler::add))
        .route("/edit/:id", get(handler::edit_ui).post(handler::edit))
        .route("/del/:id", get(handler::del))
        .route("/real_del/:id", get(handler::real_del))
        .route("/tran", get(handler::tran_ui).post(handler::tran))
        .with_state(AppState {
            pool: Arc::new(pool)
        });

    let listener = tokio::net::TcpListener::bind(&cfg.web.addr).await.unwrap();

    tracing::info!("服务器运行于: {}", listener.local_addr().unwrap());

    axum::serve(listener, app).with_graceful_shutdown(shutdown_signal()).await.unwrap()
}


async fn shutdown_signal() {
    let ctrl_c = async {
        signal::ctrl_c()
            .await
            .expect("failed to install Ctrl+C handler");
    };

    #[cfg(unix)]
    let terminate = async {
        signal::unix::signal(signal::unix::SignalKind::terminate())
            .expect("failed to install signal handler")
            .recv()
            .await;
    };
    #[cfg(not(unix))]
    let terminate = std::future::pending::<()>();
    tokio::select! {
        _ = ctrl_c => {},
        _ = terminate => {},
    }
    tracing::info!("signal received, starting graceful shutdown");
}